#ifndef __DATABASE_HPP__
#define __DATABASE_HPP__

#include "passwd.hpp"
#include "shadow.hpp"

#include <stdio.h>
#include <time.h>
#include <sys/types.h>

#define MAX_PATH		1024
#define BLOCK_SIZE	1024
#define LOCK_TRIES	  15					// Tries
#define LOCK_SLEEP	   1					// Seconds

template <class TEntry> class CDatabase
{
	public:
		CDatabase(const char *, bool, bool);
		virtual ~CDatabase();
	
	public:
		virtual int GetError() { return iErrorCode; };
		virtual bool FindUser(char *);
		virtual gid_t GetGroup(char *);
		virtual bool ChangePassword(char *, char *, int, int, int, int);
		virtual bool DeleteEntry(char *);
		virtual bool UpdateDatabase();
		virtual char *GetPassword(char *);
	
	private:
		bool CreateBackup();
		bool CreateTemp();
		bool Lock();
		bool LockNow();
		void Unlock();

	private:
		char szDatabaseName[MAX_PATH];
		char szBackupName[MAX_PATH];
		char szTempName[MAX_PATH];	
	
		bool fgLocked;
		bool fgUseLckpwdf;
	
		FILE *fpDatabase;
	
	protected:
		TEntry **ppEntry;
	
		bool fgModified;
		bool fgReadOnly;	
	
		int iEntries;
		int iErrorCode;
};

#define SHADOW_FILE "/etc/shadow"
#define PASSWD_FILE "/etc/passwd"

class CShadowDatabase : public CDatabase <class CShadowEntry>
{
	public:
		CShadowDatabase(bool fgReadOnly = true);
	
	public:
		bool AddEntry(char *);
};

class CPasswdDatabase : public CDatabase <class CPasswdEntry>
{
	public:
		CPasswdDatabase(bool fgReadOnly = true);
	
	public:
		bool AddEntry(char *, unsigned int, unsigned int, char *, char *, char *);
		unsigned int GetMaxUserID();
};

// The above two classes are used only for visualisation of the object
// relationship

#endif // __DATABASE_HPP__