#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>

#include "globals.h"
#include "sockio.h"
#include "conf.hpp"

#define SERV_TCP_PORT 1099
#define CONF_FILE		 "/etc/passwdc.conf"

#define strzero(p) memset(p, 0, strlen(p))
#define memzero(p) memset(p, 0, sizeof(p))

int main(int argc, char **argv)
{
	static char szServers[512];
	
	static char szPassword[128];
	
	struct sockaddr_in client;
	struct hostent *pHost;
	
	char *pszLogin = (char *)NULL;
	char *pszTarget = (char *)NULL;
	char *pszServer;
	char *pszPassword;
	
	int sockfd;
	
	int iServerPort;
	
	int iResult = 0;
	
	int iParams = 0;
	
	char *pszConfig = getenv("PASSWDCCONF");
	bool fgError = false;

	if (argc > 1)
		for (int i = 1; i < argc; i++)
			if ('-' == argv[i][0]) {				// We have option
				switch (argv[i][1]) {
					case 'c':							// We have other config file
						if ('\0' != argv[i][2]) {	// The user has not typed space after -c
							pszConfig = &argv[i][2];
						} else {
							pszConfig = i < (argc - 1) ? argv[i + 1] : (char *)NULL;
							if (NULL == pszConfig) {
								fprintf(stderr, "Invalid configuration file specified...\n");
								fgError = true;
							}
							i += 1;
						}
					break;
					default:
						fprintf(stderr, "Invalid option: %s...\n", argv[i]);
						fgError = true;
					break;
				}
			} else {
				if (0 == iParams)
					pszTarget = argv[i];
				else if (1 == iParams)
					pszLogin = argv[i];
				else
					fgError = true;
				iParams += 1;
			}

	if ((argc < 1) || (argc > 5) || (true == fgError)) {
		printf("Usage: userdel [<target>] [<user>] [-c <config file>]\n");
		printf("Where: <target>    is the user whose account you want to delete;\n");
		printf("       <login>     is your login if you are master user for the given target.\n");
		printf("Options: -c <config file>\n");
		printf("                   use alternate config file instead the default\n");
		printf("	                  '/etc/passwdc.conf' system wide config file;\n");
		printf("Written by: <Alexander Feldman> alex@varna.net\n\n");
		return 1;
	}
	
	if (NULL == pszConfig)
		pszConfig = CONF_FILE;
	
	char *pszMessage = NULL;	
	CConfigurationFile *pConfigFile = new CConfigurationFile(pszConfig, &pszMessage);
	if (NULL == pConfigFile) {
		fprintf(stderr, "Error allocating memory...\n");
		return 1;
	}
	if (NULL != pszMessage) {
		fprintf(stderr, "%s\n", pszMessage);
		delete pConfigFile;
		return 1;
	}
	
	iServerPort = pConfigFile->GetInteger("Global", "port", SERV_TCP_PORT);
	memzero(szServers);
	strncpy(szServers, pConfigFile->GetString("Global", "targets", "localhost"), sizeof(szServers) - 1);
	
	openlog(argv[0], LOG_PID, LOG_USER);
	
	if (NULL == pszLogin)
		pszLogin = getlogin();
	if (NULL == pszTarget)
		pszTarget = pszLogin;
	
	pszPassword = getpass("Password:");
	strcpy(szPassword, pszPassword);
	strzero(pszPassword);

	pszServer = strtok(szServers, ";,");
	do {
		pszServer += strspn(pszServer, " \t");			
		printf("Deleting %s on %s\n", pszTarget, pszServer);
	
		if ((sockfd = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
			printf("Socket error: %s\n", strerror(errno));
			continue;
		}

		pHost = gethostbyname(pszServer);
		if (NULL == pHost) {
			printf("Resolving error: %s\n", pszServer);
			close(sockfd);
			continue;
		}
		memset(&client, 0, sizeof(client));
		client.sin_family = AF_INET;
		client.sin_addr.s_addr = *((unsigned long int *)(pHost->h_addr));
		client.sin_port = htons(iServerPort);
		if (connect(sockfd, (struct sockaddr *)&client, sizeof(client)) < 0) {
			printf("Connect error: %s\n", strerror(errno));
			close(sockfd);
			continue;
		}

		unsigned char c = '\0';
		read_byte(sockfd, &c);
		if (WELCOME != c) {
			printf("Invalid response from server.\n");
			close(sockfd);
			continue;
		} 
		if (((int)strlen(pszLogin) + 1 != write_string(sockfd, pszLogin)) ||
			 ((int)strlen(szPassword) + 1 != write_string(sockfd, szPassword))) {
			printf("Write error.\n");
			close(sockfd);
			continue;
		} 
		read_byte(sockfd, &c);			
		switch (c) {
			case GO_AHEAD:
				write_byte(sockfd, ACTION_DELETEUSER);
				if ((int)strlen(pszTarget) + 1 != write_string(sockfd, pszTarget)) {
					printf("Write error.\n");
					close(sockfd);
					continue;
				}
				read_byte(sockfd, &c);
				switch (c) {
					case OK:
						printf("User successfully deleted.\n");
					break;
					case INVALID_USER:
						printf("Invalid target specified.\n");
					break;
					case ERROR:
						printf("User is not deleted. Server returned error.\n");
					break;
					case NO_RIGHTS:
						printf("Access denied.\n");
					break;
					default:
						printf("Invalid response from server.\n");
				}
			break;
			case LOGIN_FAILED:
				printf("Incorrect login or password for %s on %s.\n", pszLogin, pszServer);				
			break;
			default:
				printf("Invalid response from server.\n");
		}
		close(sockfd);
	} while (NULL != (pszServer = strtok(NULL, ";,")));

	closelog();

	strzero(szPassword);

	delete pConfigFile;
	
	return iResult;
}